/* grTOGL2.c -
 *
 *     ********************************************************************* 
 *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
 *     * Permission to use, copy, modify, and distribute this              * 
 *     * software and its documentation for any purpose and without        * 
 *     * fee is hereby granted, provided that the above copyright          * 
 *     * notice appear in all copies.  The University of California        * 
 *     * makes no representations about the suitability of this            * 
 *     * software for any purpose.  It is provided "as is" without         * 
 *     * express or implied warranty.  Export of this software outside     * 
 *     * of the United States of America may require an export license.    * 
 *     *********************************************************************
 *
 * This file contains additional functions to manipulate an X
 * color display.  Included here are rectangle drawing and color map
 * loading.
 */

#include <stdio.h>
char *getenv();

#include <GL/gl.h>
#include <GL/glx.h>

#include "magic/tclmagic.h"
#include "misc/magic.h"
#include "textio/textio.h"
#include "utils/geometry.h"
#include "graphics/glyphs.h"
#include "windows/windows.h"
#include "graphics/graphics.h"
#include "graphics/graphicsInt.h"
#include "grTOGLInt.h"

unsigned long	     grPixels[256];
unsigned long	     grPlanes[256];
extern   char	     *DBWStyleType;
extern   XVisualInfo *grVisualInfo;
extern   GLXContext  grXcontext;
extern   Display     *grXdpy;


/*---------------------------------------------------------
 * GrOGLSetCMap (pmap)
 *
 * Results:	None.
 *
 * Side Effects:
 *	The values in the color map are set from the array indicated
 *	by pmap. X color cells are allocated if this display has
 *	more than 1 plane.
 *
 * Errors:		None.
 *
 *---------------------------------------------------------
 */

Void
GrTOGLSetCMap (pmap)
    char *pmap;			/* A pointer to 256*3 bytes containing the
				 * new values for the color map.  The first
				 * three values are red, green, and blue
				 * intensities for color 0, and so on.
				 */
{
    if (GrLoadStyles(DBWStyleType, ".", SysLibPath))  MainExit();
}

TOGLLine grtoglLines[TOGL_BATCH_SIZE];
int grtoglNbLines = 0;
TOGLRect grtoglRects[TOGL_BATCH_SIZE];
int grtoglNbRects = 0;

/*---------------------------------------------------------
 * grtoglDrawLines:
 *	This routine draws a batch of lines.
 *
 * Results:	None.
 *
 * Side Effects:
 *	Draw a bunch of lines.
 *---------------------------------------------------------
 */
Void
grtoglDrawLines(lines, nb)
    TOGLLine lines[];
    int nb;
{

#ifdef OGL_SERVER_SIDE_ONLY

    int i;

    glBegin(GL_LINES);
    for (i = 0; i < nb; i++)
    {
#ifdef THREE_D
	glVertex3i(lines[i].r_ll.p_x, lines[i].r_ll.p_y, lines[i].r_ll.p_z);
	glVertex3i(lines[i].r_ur.p_x, lines[i].r_ur.p_y, lines[i].r_ur.p_z);
#else
	glVertex2i(lines[i].r_ll.p_x, lines[i].r_ll.p_y);
	glVertex2i(lines[i].r_ur.p_x, lines[i].r_ur.p_y);
#endif
    }
    glEnd();

#else

#ifdef THREE_D
    glVertexPointer(3, GL_INT, 0, (GLvoid *)lines);
#else
    glVertexPointer(2, GL_INT, 0, (GLvoid *)lines);
#endif
    glDrawArrays(GL_LINES, 0, nb << 1);

#endif
}

/*---------------------------------------------------------
 * grtoglDrawLine:
 *	This routine draws a line.
 *
 * Results:	None.
 *
 * Side Effects:
 *	Draw a line for (x1, y1) to (x2, y2) inclusive.
 *---------------------------------------------------------
 */
Void
grtoglDrawLine (x1, y1, x2, y2)
    int x1, y1;			/* Screen coordinates of first point. */
    int x2, y2;			/* Screen coordinates of second point. */
{
    if (grtoglNbLines == TOGL_BATCH_SIZE) GR_TOGL_FLUSH_LINES();
    grtoglLines[grtoglNbLines].r_ll.p_x = x1;
    grtoglLines[grtoglNbLines].r_ll.p_y = y1;
    grtoglLines[grtoglNbLines].r_ur.p_x = x2;
    grtoglLines[grtoglNbLines].r_ur.p_y = y2;

#ifdef THREE_D
    grtoglLines[grtoglNbLines].r_ll.p_z = grHeight;
    grtoglLines[grtoglNbLines].r_ur.p_z = grHeight;
#endif

    grtoglNbLines++;
}

/*---------------------------------------------------------
 * grtoglFillRects:
 *	This routine draws a bunch of solid rectangles.
 *
 * Results:	None.
 *
 * Side Effects:
 *	Drawing.
 *---------------------------------------------------------
 */

Void
grtoglFillRects(rects, nb)
    TOGLRect rects[];
    int nb;
{

#ifdef OGL_SERVER_SIDE_ONLY

    int i;

#ifdef THREE_D
    if (grtoglVolumeRender)
    {
        for (i = 0; i < nb; i++)
	{
	    glBegin(GL_POLYGON);
	    glVertex3i(rects[i].r_ll.p_x, rects[i].r_ll.p_y, rects[i].r_ll.p_z);
	    glVertex3i(rects[i].r_ll.p_x, rects[i].r_ur.p_y, rects[i].r_ll.p_z);
	    glVertex3i(rects[i].r_ur.p_x, rects[i].r_ur.p_y, rects[i].r_ll.p_z);
	    glVertex3i(rects[i].r_ur.p_x, rects[i].r_ll.p_y, rects[i].r_ll.p_z);
	    glEnd();
	}
    }
    else
#endif
        for (i = 0; i < nb; i++)
	    glRecti(rects[i].r_ll.p_x, rects[i].r_ll.p_y,
			rects[i].r_ur.p_x, rects[i].r_ur.p_y);
#else

#ifdef THREE_D
    glVertexPointer(3, GL_INT, 0, (GLvoid *)rects);
#else
    glVertexPointer(2, GL_INT, 0, (GLvoid *)rects);
#endif
    glDrawArrays(GL_QUADS, 0, nb << 2);

#endif
}

/*---------------------------------------------------------
 * grtoglFillRect:
 *	This routine draws a solid rectangle.
 *
 * Results:	None.
 *
 * Side Effects:
 *	Drawing.
 *---------------------------------------------------------
 */

Void
grtoglFillRect(r)
    register Rect *r;	/* Address of a rectangle in screen
			 * coordinates.
			 */
{
    if (grtoglNbRects == TOGL_BATCH_SIZE) GR_TOGL_FLUSH_RECTS();
    grtoglRects[grtoglNbRects].r_ll.p_x = r->r_ll.p_x;
    grtoglRects[grtoglNbRects].r_ll.p_y = r->r_ll.p_y;

    grtoglRects[grtoglNbRects].r_ur.p_x = r->r_ur.p_x;
    grtoglRects[grtoglNbRects].r_ur.p_y = r->r_ur.p_y;

#ifdef THREE_D
    grtoglRects[grtoglNbRects].r_ll.p_z = grHeight;
    grtoglRects[grtoglNbRects].r_ur.p_z = grHeight;
#endif

#ifndef OGL_SERVER_SIDE_ONLY
    grtoglRects[grtoglNbRects].r_ul.p_x = r->r_ll.p_x;
    grtoglRects[grtoglNbRects].r_ul.p_y = r->r_ur.p_y;

    grtoglRects[grtoglNbRects].r_lr.p_x = r->r_ur.p_x;
    grtoglRects[grtoglNbRects].r_lr.p_y = r->r_ll.p_y;

#ifdef THREE_D
    grtoglRects[grtoglNbRects].r_ul.p_z = grHeight;
    grtoglRects[grtoglNbRects].r_lr.p_z = grHeight;
#endif
#endif

    grtoglNbRects++;
}

#ifdef THREE_D
/*---------------------------------------------------------
 * GrDrawSide:
 *	This routine draws a solid rectangle at a vertical
 *	orientation to a tile.
 *	Like GrClear(), it is only defined for the TOGL
 *	option.
 *
 * Results:	None.
 *
 * Side Effects:
 *	Drawing.
 *---------------------------------------------------------
 */

void
GrDrawSide(x1, y1, x2, y2)
    int x1, y1;
    int x2, y2;
{
    /* Line translates to a vertical plane in a 3D view */
    if (grtoglVolumeRender)
    {
	if (grtoglNbRects == TOGL_BATCH_SIZE) GR_TOGL_FLUSH_RECTS();
	grtoglRects[grtoglNbRects].r_ll.p_x = x1;
	grtoglRects[grtoglNbRects].r_ll.p_y = y1;
	grtoglRects[grtoglNbRects].r_ll.p_z = grHeight;

	grtoglRects[grtoglNbRects].r_ur.p_x = x2;
	grtoglRects[grtoglNbRects].r_ur.p_y = y2;
	grtoglRects[grtoglNbRects].r_ur.p_z = grHeight + grThick;

#ifndef OGL_SERVER_SIDE_ONLY
	grtoglRects[grtoglNbRects].r_ul.p_x = x1;
	grtoglRects[grtoglNbRects].r_ul.p_y = y1;
	grtoglRects[grtoglNbRects].r_ul.p_z = grHeight + grThick;

	grtoglRects[grtoglNbRects].r_lr.p_x = x2;
	grtoglRects[grtoglNbRects].r_lr.p_y = y2;
	grtoglRects[grtoglNbRects].r_lr.p_z = grHeight;
#endif
	grtoglNbRects++;
    }
}
#endif /* THREE_D */

#ifdef NONMANHATTAN

/*---------------------------------------------------------
 * grtoglFillPolygon:
 *	This routine draws a solid polygon
 *
 * Results:     None.
 *
 * Side Effects:
 *	Drawing.
 *---------------------------------------------------------
 */

Void
grtoglFillPolygon(tp, np)
    Point *tp;
    int np;
{
    int i;

    glBegin(GL_POLYGON);
    for (i = 0; i < np; i++)
#ifdef THREE_D
	glVertex3i(tp[i].p_x, tp[i].p_y, grHeight);
#else
	glVertex2i(tp[i].p_x, tp[i].p_y);
#endif
    glEnd();
}

#endif NONMANHATTAN

