# console.tcl:
#
#  Defines Procedures which should be defined within the console
#  (master) interpreter (i.e., tkcon, if it is executed as a separate
#  interpreter).  Note that this can always be worked around via the
#  "tkcon master" or "tkcon slave" commands, but this method organizes
#  things in a more natural way.

wm geometry . 120x16+150-50

proc magiccolor { ch } {
    tkcon slave slave magic::magiccolor %ch
}

proc replaceprompt { ch } {
    tkcon set ::tkcon::OPT(prompt1) "$ch "
    .text delete prompt.last-2c
    .text insert [.text index prompt.last-1c] $ch {prompt}
    .text mark set output [.text index output-1c]
}

# This procedure repaints the magic console to match the magic colormap.
# It's only called if magic is run in 8-bit (PseudoColor) visual mode.
# This is called from inside magic (graphics/grTk1.c).

proc repaintconsole {} {
    puts stdout "Repainting console in magic layout window colors"
    set bgcolor [magic::magiccolor -]

    tkcon set ::tkcon::COLOR(bg) $bgcolor
    tkcon set ::tkcon::COLOR(blink) [magiccolor Y]
    tkcon set ::tkcon::COLOR(cursor) [magiccolor K]
    tkcon set ::tkcon::COLOR(disabled) [magiccolor k]
    tkcon set ::tkcon::COLOR(proc) [magiccolor G]
    tkcon set ::tkcon::COLOR(var) [magiccolor p]
    tkcon set ::tkcon::COLOR(prompt) [magiccolor w]
    tkcon set ::tkcon::COLOR(stdin) [magiccolor K]
    tkcon set ::tkcon::COLOR(stdout) [magiccolor B]
    tkcon set ::tkcon::COLOR(stderr) [magiccolor R]

    .text configure -background $bgcolor
    .text configure -foreground [magiccolor K]
    .text configure -insertbackground [magiccolor K]

    .text tag configure var -background  $bgcolor
    .text tag configure blink -background  $bgcolor
    .text tag configure find -background  $bgcolor
    .text tag configure stdout -background  $bgcolor
    .text tag configure stderr -background  $bgcolor
    .text tag configure proc -background  $bgcolor
    .text tag configure prompt -background  $bgcolor

    .text tag configure var -foreground [magiccolor p]
    .text tag configure blink -foreground [magiccolor Y]
    .text tag configure find -foreground [magiccolor Y]
    .text tag configure stdout -foreground [magiccolor B]
    .text tag configure stderr -foreground [magiccolor R]
    .text tag configure proc -foreground [magiccolor G]
    .text tag configure prompt -foreground [magiccolor w]

    .sy configure -background $bgcolor
}

